/*
  The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "MPL"); you may not use this file except in compliance
  with the Licens. You may obtain a copy of the MPL at
  http://www.mozilla.org/MPL/

  Software distributed under the MPL is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the MPL
  for the specific language governing rights and limitations under the
  MPL.

  The Original Code is Forward Extension.

  The Initial Developer of the Original Code is Onno Ekker.
  Portions created by Onno Ekker <o.e.ekker@gmail.com> are Copyright (c)
  2008. All Rights Reserved.

  Alternatively, the contents of this file may be used under the terms
  terms of the GNU General Public License (the "GPL"), in which case the
  provisions of the GPL are applicable instead of those above. If you
  wish to allow use of your version of this file only under the terms of
  the GPL and not to allow others to use your version of this file under
  the MPL, indicate your decision by deleting the provisions above and
  replace them with the notice and other provisions required by the GPL.
  If you do not delete the provisions above, a recipient may use your
  version of this file under either the MPL or the GPL.
*/

function myLogToErrorConsole(aMessage, aSourceName, aSourceLine, aLineNumber, 
                             aColumnNumber, aFlags, aCategory)
{
  var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
  if (aFlags == null)
    consoleService.logStringMessage(aMessage);
  else
  {
    var scriptError = Components.classes["@mozilla.org/scripterror;1"]
                              .createInstance(Components.interfaces.nsIScriptError);
    scriptError.init(aMessage, aSourceName, aSourceLine, aLineNumber, 
                     aColumnNumber, aFlags, aCategory);
    consoleService.logMessage(scriptError);
  }
}

// forward-button

function updateForwardButtonMenu(event)
{
  var forwardType = 0;
  var forwardInlineMenuItem = document.getElementById('button-forwardAsMenu-forwardInline');
  var forwardAsAttachmentMenuItem = document.getElementById('button-forwardAsMenu-forwardAsAttachment');

  function ucfirst(menuItem)
  {
    var origLabel = menuItem.getAttribute('label');
    var newLabel = origLabel.replace(/^(\S)/, function(t, a) { return a.toLocaleUpperCase(); });
    if (newLabel != origLabel) menuItem.setAttribute('label', newLabel);
  }

  // mail.forward_message_mode could be 1, if the user migrated from 4.x
  // 1 (forward as quoted) is obsolete, so we treat is as forward inline
  // since that is more like forward as quoted then forward as attachment
  try {
    forwardType = gPrefBranch.getIntPref('mail.forward_message_mode');
  }
  catch (ex) {
    myLogToErrorConsole(ex, 'chrome://forward/content/forward.js',
                        '    forwardType = gPrefBranch.getIntPref(\'mail.forward_message_mode\');',
                        65, 0, 2, 'javascript');
    forwardType = 2; // Let's make Inline the default
  }

  if (forwardInlineMenuItem) {
    forwardInlineMenuItem.setAttribute('default', (forwardType != 0));
    ucfirst(forwardInlineMenuItem);
  }
  if (forwardAsAttachmentMenuItem) {
    forwardAsAttachmentMenuItem.setAttribute('default', (forwardType == 0));
    ucfirst(forwardAsAttachmentMenuItem);
  }
}

// Hightlight default forward mode
window.addEventListener("load", updateForwardButtonMenu, false);

var myPrefObserver =
{
  register: function()
  {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefService);
    this._branch = prefService.getBranch("mail.forward_message_mode");
    this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this._branch.addObserver("", this, false);
  },

  unregister: function()
  {
    if(!this._branch) return;
    this._branch.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData)
  {
    if(aTopic != "nsPref:changed") return;
    updateForwardButtonMenu();
  }
}

// Watch for preference changes
myPrefObserver.register();

// threadPane and messagePane menu menuitem

function SetupForwardAsMenuMenuItem(menuID, numSelected, forceHide)
{
  ShowMenuItem(menuID, (numSelected <= 1) && !forceHide);
  EnableMenuItem(menuID, (numSelected == 1));
}

// threadPane

function forward_fillThreadPaneContextMenu()
{
  var numSelected = GetNumSelectedMessages();
  SetupForwardAsMenuMenuItem("threadPaneContext-forwardAsMenu", numSelected, false);
}

// messagePane

function forward_fillMessagePaneContextMenu()
{
  var message = GetLoadedMessage();
  var numSelected = (message) ? 1 : 0;

  // don't show mail items for links/images, just show related items.
  var hideMailItems = gContextMenu.onImage || gContextMenu.onLink;
  SetupForwardAsMenuMenuItem("messagePaneContext-forwardAsMenu", numSelected, (numSelected == 0 || hideMailItems));
}

function installListeners(event)
{
  var el;
  el = document.getElementById("threadPaneContext");
  if (el) el.addEventListener("popupshowing", forward_fillThreadPaneContextMenu, false);

  el = document.getElementById("messagePaneContext");
  if (el) el.addEventListener("popupshowing", forward_fillMessagePaneContextMenu, false);
}

function uninstallListeners(event)
{
  var el;
  el = document.getElementById("threadPaneContext");
  if (el) el.removeEventListener("popupshowing", forward_fillThreadPaneContextMenu, false);

  el = document.getElementById("messagePaneContext");
  if (el) el.removeEventListener("popupshowing", forward_fillMessagePaneContextMenu, false);
}

window.addEventListener("load", installListeners, false);
window.addEventListener("unload", uninstallListeners, false);
